#File: FFT.tcl
#Syscomp USB Oscilloscope GUI
#Spectrum Analysis Toolbox

#JG
#Copyright 2007 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#The fourier math package is the basis for spectrum analysis
if {$osType=="windows"} {
	lappend auto_path "$include/MathWin"
} else {
	lappend auto_path "$include/MathLin"
}
package require math::fourier 1.0.1

#Add a menubar item to the main window to access the FFT
.menubar.tools.menu add command	\
	-label "Spectrum Analysis"	\
	-command createFFT
.menubar.tools.menu add separator

#FFT Display Geometry
set fftCanvasSize 500
set fftXBorder 40
set fftXLength [expr {$fftCanvasSize-2*$fftXBorder}]
set fftYBorder 20
set fftYLength [expr {$fftCanvasSize-2*$fftYBorder }]

#FFT Interface Control Variables
set fftScale linear
set fftWindow rectangular
set fftSource a
set fftEnable 0

#Create Spectrum Analysis Window
#----------------
#This procedure creates the spectrum analysis window, initializes all widgets,
#and performs the geometry placement.  It also sets the fftEnable global variable
#which signals that the FFT analysis is running.
proc createFFT {} {
	global fftEnable	\
		fftCanvasSize	\
		fftXBorder	\
		fftXLength	\
		fftYBorder	\
		fftYLength	\
		freqPos

	#Check to see if the FFT windows is already running, if so, raise it
	if {[winfo exists .fft]} {
		raise .fft
		return
	}

	#Create the FFT Window
	toplevel .fft
	wm resizable .fft 0 0
	wm title .fft "Frequency Spectrum"
	
	label .fft.title	\
	-text "Frequency Spectrum Analysis"	\
	-font {-weight bold -size 14}
	
	#Vertical Scale Controls
	frame .fft.scale	\
		-relief groove	\
		-borderwidth 2

	label .fft.scale.title	\
		-text "Vertical Scale"	\
		-font {-weight bold -size 10}


	radiobutton .fft.scale.linear	\
		-text "Linear"	\
		-value "linear"	\
		-variable fftScale

	radiobutton .fft.scale.log	\
		-text "Logarithmic"	\
		-value "log"			\
		-variable fftScale
		
	grid .fft.scale.title -row 0 -column 0 -sticky we
	grid .fft.scale.linear -row 1 -column 0 -sticky w
	grid .fft.scale.log -row 2 -column 0 -sticky w

	#FFT Windowing Controls
	frame .fft.window	\
		-relief groove	\
		-borderwidth 2
		
	label .fft.window.title	\
		-text "FFT Window"	\
		-font {-weight bold -size 10}

	radiobutton .fft.window.rectangular	\
		-text "Rectangular"	\
		-value rectangular	\
		-variable fftWindow
		
	radiobutton .fft.window.hamming	\
		-text "Hamming"	\
		-value hamming	\
		-variable fftWindow
		
	grid .fft.window.title -row 0 -column 0 -sticky we
	grid .fft.window.rectangular -row 1 -column 0 -sticky w
	#grid .fft.window.hamming -row 2 -column 0 -sticky w

	#FFT Data Source Controls
	frame .fft.source	\
		-relief groove	\
		-borderwidth 2
		
	label .fft.source.title	\
		-text "FFT Source"	\
		-font {-weight bold -size 10}
		
	radiobutton .fft.source.a	\
		-text "Channel A"		\
		-value a			\
		-variable fftSource
		
	radiobutton .fft.source.b	\
		-text "Channel B"		\
		-value b			\
		-variable fftSource
		
	grid .fft.source.title -row 0 -column 0 -sticky we
	grid .fft.source.a -row 1 -column 0 -sticky w
	grid .fft.source.b -row 2 -column 0 -sticky w
	
	#Sampling Information
	frame .fft.sample	\
		-relief groove	\
		-borderwidth 2
		
	label .fft.sample.rateLabel	\
		-text "Effective Sampling Rate"
		
	label .fft.sample.rate	\
		-relief sunken	\
		-textvariable fftSampleRate
		
	label .fft.sample.numLabel	\
		-text "Number of Samples"
	
	label .fft.sample.numSamples	\
		-relief sunken	\
		-textvariable fftSamples
		
	grid .fft.sample.rateLabel -row 0 -column 0
	grid .fft.sample.rate -row 0 -column 1
	grid .fft.sample.numLabel -row 1 -column 0
	grid .fft.sample.numSamples -row 1 -column 1
	
	#FFT Display
	canvas .fft.display	\
	-width $fftCanvasSize		\
	-height $fftCanvasSize	\
	-background black

	#Draw the Axes
	.fft.display create line	\
		$fftXBorder $fftYBorder $fftXBorder [expr {$fftYBorder + $fftYLength}]	\
		-fill white
	.fft.display create line	\
		$fftXBorder [expr {$fftYBorder+$fftYLength}] [expr {$fftXBorder+$fftXLength}] [expr {$fftYBorder+$fftYLength}]	\
		-fill white
	.fft.display create text	\
		[expr {$fftXBorder-4}] [expr {$fftYBorder+$fftYLength + 6}]	\
		-fill white	\
		-text "0"
		
	#Draw the frequency cursor
	.fft.display create line	\
		[expr {$fftXBorder+$fftXLength/2}] $fftYBorder	\
		[expr {$fftXBorder+$fftXLength/2}] [expr {$fftYBorder+$fftYLength}]	\
		-fill green	\
		-tag freqCursor
	.fft.display create polygon	\
		[expr {$fftXBorder+$fftXLength/2-5}] $fftYBorder	\
		[expr {$fftXBorder+$fftXLength/2+5}] $fftYBorder	\
		[expr {$fftXBorder+$fftXLength/2}] [expr {$fftYBorder+7}]	\
		[expr {$fftXBorder+$fftXLength/2-3}] $fftYBorder	\
		-fill green	\
		-tag freqCursor
	
	#Initialize Frequency Cursor Position
	set freqPos [expr {$fftXBorder+$fftXLength/2}]
	.fft.display bind freqCursor <Button-1> {markFreqStart freqCursor %x}
	.fft.display bind freqCursor <B1-Motion> {moveFreqCursor %x}
		
	#Arrange the FFT user interface
	grid .fft.title -row 0 -column 0 -columnspan 5 -sticky w
	grid .fft.display -row 1 -column 0 -columnspan 5
	grid .fft.scale -row 2 -column 0
	grid .fft.source -row 2 -column 1
	grid .fft.window -row 2 -column 2
	grid .fft.sample -row 2 -column 3

	bind .fft <Destroy> {set fftEnable 0}
	set fftEnable 1

}

#Update Spectrum Display
#----------------
#This procedure is called each time there is new data available from the scope.  This
#is where we perform the actual FFT number crunching and plot the spectrum on the
#display.
proc updateFFT {} {
	global exportData	\
		fftYLength	\
		fftXLength	\
		fftXBorder	\
		fftYBorder	\
		sampleRate	\
		fft	\
		fftCanvasSize	\
		fftScale	\
		fftSource	\
		fftSamples	\
		fftSampleRate	\
		burstIncrement
	
	#Determine which channel data we will be processing
	if {$fftSource == "a" } {
		set fftData [lindex $exportData 1]
		set stepSize [lindex $exportData 4]
	} else {
		set fftData [lindex $exportData 2]
		set stepSize [lindex $exportData 5]
	}
	
	#Convert the data into voltage values
	set norm {}
	foreach datum $fftData {
		lappend norm [expr (128-$datum)*$stepSize]
	}
	
	#Determine the number of samples
	set normLength [llength $norm]
	
	#Determine the length of the FFT to the next higher power of 2
	set powTwo [expr {log10($normLength)/log10(2)}]
	set powTwo [expr {ceil($powTwo)}]
	set fftLength [expr {pow(2,$powTwo)}]
	set fftSamples $fftLength
	
	#Pad the data with zeros to ensure that we have a record length
	#that is a power of 2
	while {[llength $norm] < $fftLength} {lappend norm 0}
	
	#Calculate the effective sampling rate by dividing the hardware sampling rate
	#by the burstIncrement.  The burstIncrement is used to subsample the data.
	set fftSampleRate [expr {20.0E6/pow(2,[sampleConstant])/$burstIncrement}]
	
	#Format the effective sampling rate for display purposes
	if {$fftSampleRate >=1E6} {
		set fftSampleRate [expr {$fftSampleRate/1.0E6}]
		set fftSampleRate [format "%.1f" $fftSampleRate]
		set fftSampleRate "$fftSampleRate MS/s"
	} elseif {$fftSampleRate >=1E3} {
		set fftSampleRate [expr {$fftSampleRate/1.0E3}]
		set fftSampleRate [format "%.1f" $fftSampleRate]
		set fftSampleRate "$fftSampleRate kS/s"
	} else {
		set fftSampleRate [format "%.1f" $fftSampleRate]
		set fftSampleRate "$fftSampleRate S/s"
	}
	
	#Crunch the numbers: FFT analysis
	set fft [::math::fourier::dft $norm]
	
	#Extract the spectrum data from the FFT data
	set spectrum {}
	set halfLength [expr {([llength $fft]/2)+1}]
	for {set index 0} {$index < $halfLength} {incr index} {
		#Read the complex number list from the FFT data
		set complex [lindex $fft $index]
		#Seperate the complex number into the real and complex parts
		set r [lindex $complex 0]
		set i [lindex $complex 1]
		#Compute the power sprectrum
		set temp [expr {sqrt(($r*$r)+($i*$i))}]
		#Except for the DC value, we double the result
		if { $index > 0} {
			set temp [expr {$temp*2.0}]
		}
		lappend spectrum $temp
	}
	
	#Linear Scaling
	set spectrumLinear {}
	foreach element $spectrum {
		lappend spectrumLinear [expr {$element/$fftLength}]
	}

	#Logarithmic Scaling
	set spectrumLog {}
	if {$fftScale == "log"} {
		#The A/D Step Size is used as the 0dB reference
		set ref $stepSize
		#Compute the logarithmically scaled spectrum
		foreach element $spectrumLinear {
			if {$element < $stepSize} {
				#Make sure that we don't take the log of zero and discard
				#numbers from the FFT analysis that are smaller than the A/D
				#step size.
				set element $stepSize
			} 
			#Compute the power spectrum
			lappend spectrumLog [expr {20.0*log10($element/$ref)}]
		}
	}
	
	#Scale the data for the screen and create an array for plotting
	set plotData {}
	if {$fftScale == "linear"} {
		set linMax [expr {ceil(128.0*$stepSize)}]
		foreach datum $spectrumLinear {
			lappend plotData [expr {$fftYBorder+$fftYLength-($datum/$linMax*$fftYLength)}]
		}
	} else {
		#The maximum dynamic range is 48 dB (20*log(256)), so the log scale span is 50 dB
		foreach datum $spectrumLog {
			lappend plotData [expr {$fftYBorder+$fftYLength-$fftYLength*($datum/50.0)}]
		}
	}
	
	#Scale the spectrum to take up the full X-Axis, the x-axis labelling is taken
	#care of by the cursor routines.
	set xLength [llength $plotData]
	set xLength [expr {$xLength*1.0}]
	set drawSpectrum {}
	for {set index 0} {$index < $xLength} {incr index} {
		#Calculate the relative position of this sample on the x-axis
		set x [expr {(($index+1)/$xLength*$fftXLength)+$fftXBorder}]
		#Pull the corresponding power spectrum value from the list
		set point [lindex $plotData $index]
		#Save this x,y pair for plotting
		lappend drawSpectrum $x $point
	}
	
	#Spectrum colors should match the coloring of the traces in the main window.
	if {$fftSource == "a"} {
		set spectrumColor red
	} else {
		set spectrumColor blue
	}
	
	#Plot the FFT data on the spectrum display
	.fft.display delete fftPlot
	.fft.display create line	\
		$drawSpectrum	\
		-fill $spectrumColor	\
		-tag fftPlot
		
	#Show the reference for the logarithmic scale
	if {$fftScale == "log"} {
		.fft.display delete fftRef
		set refText "0dB = $ref V"
		.fft.display create text	\
			[expr {$fftXBorder+$fftXLength/2.0}] [expr {$fftYBorder/2.0}]	\
			-fill white	\
			-tag fftRef	\
			-text $refText
	} else {
		#Linear display, remove the log reference
		.fft.display delete fftRef
	}
	
	#Update the Y-Axis
	if {$fftScale == "linear"} {
		fftLinearScale
	} else {
		fftLogScale
	}
	
	#Copy power spectrum data to fft global variable for cursor processing
	if {$fftScale =="linear"} {
		set fft $spectrumLinear
	} else {
		set fft $spectrumLog
	}
	
	#Update the cursors since the values may have changed since the cursor was moved last
	updateFFTCursor
	
}

#Draw Linear Y Axis Markings
#----------------
#This procedure is called to annotate the y-axis with linear tick marks.  The maximum value is calculated
#automatically based on the current vertical sensitivity of the scope.  The intermediate tick marks are then
#calculated automatically.
proc fftLinearScale {} {
	global fftSource	\
		exportData	\
		fftYLength	\
		fftYBorder	\
		fftXBorder

	#Determine the data source and read out the A/D step size
	if {$fftSource == "a" } {
		set stepConst [lindex $exportData 4]
	} else {
		set stepConst [lindex $exportData 5]
	}
	
	#Determine the maximum value that can be measured at this vertical sensitivity
	set maxValue [expr {ceil(128.0*$stepConst)}]
	
	#Update the Y-Axis Tick Marks
	.fft.display delete fftYAxis
	for {set i 0} {$i <= $maxValue} {incr i} {
		set y [expr {$fftYBorder+$fftYLength-($fftYLength*$i/$maxValue)}]
		.fft.display create text	\
			[expr {$fftXBorder-($fftXBorder/2.0)}] $y	\
			-text $i	\
			-fill white	\
			-tag fftYAxis
		.fft.display create line	\
			[expr {$fftXBorder-2}] $y	\
			[expr {$fftXBorder+2}] $y	\
			-fill white	\
			-tag fftYAxis
	}
	
	#Units for the Y axis
	.fft.display create text \
		[expr {$fftXBorder-($fftXBorder/2.0)}] [expr {$fftYBorder/2.0}]	\
		-text {[V]}	\
		-fill white	\
		-tag fftYAxis
}

#Draw Logarithmic Y Axis Markings
#----------------
#This procedure is called to annotate the y-axis with logarithmic tick marks.  The maximum value fixed at
#48 dB due to the 8-bit vertical sensitivity of the scope.
proc fftLogScale {} {
	global fftSource	\
		exportData	\
		fftYLength	\
		fftYBorder	\
		fftXBorder
	
	#Draw tick marks every 10dB along the y-axis
	.fft.display delete fftYAxis
	for {set i 0} {$i <= 50} {set i [expr {$i+10}]} {
		set y [expr {$fftYBorder+$fftYLength-$fftYLength*($i/50.0)}]
		.fft.display create text	\
			[expr {$fftXBorder-($fftXBorder/2.0)}] $y	\
			-text $i	\
			-fill white	\
			-tag fftYAxis
	}

	#Units for the Y axis
	.fft.display create text \
		[expr {$fftXBorder-($fftXBorder/2.0)}] [expr {$fftYBorder/2.0}]	\
		-text {[dB]}	\
		-fill white	\
		-tag fftYAxis
	
}

#Mark Frequency Cursor Start Position
#-------------
#The procedure is used by the display when measurement cursor is moved.
#The procedure marks the starting point where the user initially
#grabs the cursor.
proc markFreqStart {cursorTag xPos} {
	global freqPos	\
		freqStart
	
	#Snapping in case the user grabbed the handle
	set snap [expr {$xPos - $freqPos}]
	set freqPos $xPos
	.fft.display move $cursorTag $snap 0
	#Mark this start postion
	set freqStart $xPos
}

#Move Frequency Cursor
#-------------------
#This procdure is called when the user drags the frequency cursor along the screen.  It updates the position
#of the cursor on the screen and calls the updateFFTCursor routine which updates the measurement readings.
proc moveFreqCursor {xPos} {
	global freqPos	\
		freqStart	\
		fftXBorder	\
		fftXLength
		
	#Make sure we don't go off the screen
	if { $xPos <= $fftXBorder } { set xPos [expr {$fftXBorder+1}] }
	if { $xPos > [expr {$fftXBorder + $fftXLength}] } {
		set xPos [expr {$fftXBorder + $fftXLength}]
	}
	#Move the cursor on the screen
	set dx [expr {$xPos - $freqStart}]
	set freqStart $xPos
	set freqPos $xPos
	.fft.display move freqCursor $dx 0
	
	updateFFTCursor
}

#Update Frequency Cursor Readings
#--------------------------------------
#This routine is called to update the frequency position reading and the amplitude reading of the frequency
#cursor.  The routine reads the amplitude data from the FFT global array and displays it on the screen.  It
#also calculates and displays the frequency position of the cursor.
proc updateFFTCursor {} {
	global freqPos	\
		fftXBorder	\
		fftXLength	\
		fftYBorder	\
		fftYLength	\
		fft		\
		exportData	\
		burstIncrement	\
		fftScale	\
		fftSource	\
		exportData
		
	#Read out the A/D step size for the selected channel.
	if {$fftSource == "a" } {
		set stepSize [lindex $exportData 4]
	} else {
		set stepSize [lindex $exportData 5]
	}
		
	#Make sure that the FFT window is still open (in case somebody closes it in the middle of the update)
	if {![info exists fft]} {return}

	#Determine the horizontal position of the cursor
	set xPos [expr {1.0*($freqPos-$fftXBorder)/$fftXLength}]
	
	#Use the horizontal position to read out the corresponding FFT amplitude data
	set fftLength [llength $fft]
	set fftIndex [expr {round($xPos*$fftLength)-1}]
	set fftValue [lindex $fft $fftIndex]
	#Format for 2 decimal places
	set fftValue [format "%.2f" $fftValue]
	
	#Draw the amplitude cross-hair indicator
	if {$fftScale == "linear"} {
		set linMax [expr {ceil(128.0*$stepSize)}]
		set h [expr {round($fftYLength+$fftYBorder-($fftValue/$linMax*$fftYLength))}]
	} else {
		set h [expr {$fftYBorder+$fftYLength-$fftYLength*($fftValue/50.0)}]
	}
	.fft.display delete fLevel
	.fft.display create line \
		[expr {$freqPos-3}] $h	\
		[expr {$freqPos+3}] $h	\
		-tag fLevel	\
		-fill green
	
	#Label the cross hair with the amplitude value
	.fft.display delete fValue
	if {$freqPos>[expr {0.75*($fftXLength+$fftXBorder)}]} {
		set textX [expr {$freqPos - 28}]
	} else {
		set textX [expr {$freqPos + 28}]
	}
	.fft.display create text	\
		$textX [expr {$h-7}]	\
		-fill green	\
		-text $fftValue	\
		-tag fValue
		
	#Calculate and display the frequency value
	set sRate [expr {20.0E6/pow(2,[sampleConstant])}]
	set fStep [expr {$sRate/(2*$burstIncrement*($fftLength-1))}]
	set fX [expr {$fStep*$fftIndex}]
	set fX [format "%.0f" $fX]
	.fft.display delete fText
	.fft.display create text	\
		$freqPos [expr {$fftYBorder+$fftYLength+7}]	\
		-fill white	\
		-text $fX	\
		-tag fText
}

#Create a fake waveform (TESTING ONLY)
#----------------
#A neat little routine that creates a fake waveform and forces the scope to udate using this data.  Used
#for offline testing.
proc fakeWave {amp period} {
	global manuallyTriggered	\
		dataARaw	\
		dataBRaw	\
		dataA	\
		dataB
	
	set dataARaw {}
	set dataBRaw {}
	set dataA {}
	set dataB {}
	
	set returnData {}
	
	for {set i 0} {$i < 512} {incr i} {
		set temp [expr {($amp*1.0)*sin(2.0*3.14*(1/($period*1.0))*$i)+128}]
		set temp [expr round($temp)]
		lappend returnData $temp
		lappend returnData 128
	}
	
	set manuallyTriggered 1
	updateOscilloscope $returnData

}