#File: cursors.tcl
#Syscomp USB Oscilloscope GUI
#Cursor Related Procedures

#JG
#File Version 1.03 January 2006
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#======================================
# Index
#--------
#	Part I - General Cursor Procedures
#	Part II - Time Cursors Procedures
#	Part III - Voltage Cursors Procedures
#	Part IV - Ground Reference Cursors Procedures
#	Part V - Trigger Cursor Procedures

#======================================
#	Part I - General Cursor Procedures
#======================================

#Mark Y Start
#-------------
#The procedure is used by the display when cursors are moved.
#The procedure marks the starting point where the user initially
#grabs the cursor.
proc markYStart { cursorTag yPos } {
	global yStart \
		va1Pos \
		va2Pos \
		vb1Pos \
		vb2Pos \
		chAGndPos \
		chBGndPos \
		trigPos \
		triggerSource
		
	set dx 0
	switch $cursorTag {
		"va1Cursor" {
			set snap [expr $yPos - $va1Pos]
			set va1Pos $yPos
			.display move $cursorTag $dx $snap
			calculateCursors
		} "va2Cursor" {
			set snap [expr $yPos - $va2Pos]
			set va2Pos $yPos
			.display move $cursorTag $dx $snap
			calculateCursors
		} "vb1Cursor" {
			set snap [expr $yPos - $vb1Pos]
			set vb1Pos $yPos
			.display move $cursorTag $dx $snap
			calculateCursors
		} "vb2Cursor" {
			set snap [expr $yPos - $vb2Pos]
			set vb2Pos $yPos
			.display move $cursorTag $dx $snap
			calculateCursors
		} "chAGnd" {
			set snap [expr $yPos - $chAGndPos]
			set chAGndPos $yPos
			.display move $cursorTag $dx $snap
			if { $triggerSource == "A" } {
				.display move trigLevelCursor $dx $snap
				set trigPos [expr $trigPos + $snap]
			}
		} "chBGnd" {
			set snap [expr $yPos - $chBGndPos]
			set chBGndPos $yPos
			.display move  $cursorTag $dx $snap
			if { $triggerSource == "B"} {
				.display move trigLevelCursor $dx $snap
				set trigPos [expr $trigPos + $snap]
			}
		} "trigLevelCursor" {
			set snap [expr $yPos - $trigPos]
			set trigPos $yPos
			.display move $cursorTag $dx $snap
		} 
	}
	
	set yStart $yPos
}

#Mark X Start
#-------------
#This procedure  is used by the display when the time cursor is
#moved.  This procedure marks the starting point where the user
#initially grabbed the cursor.
proc markXStart { cursorTag xPos } {
	global xStart \
		timePos \
		t1Pos \
		t2Pos		\
		xPlotStart	\
		xPlotEnd
	
	set dy 0
	
	#Check to see if we have gone off of the screen
	if { $xPos > $xPlotEnd} {set xPos $xPlotEnd}
	if { $xPos < $xPlotStart} { set xPos $xPlotStart}
	
	switch $cursorTag {
		"timePosCursor" {
			set snap [expr $xPos - $timePos]
			set timePos $xPos
			.display move $cursorTag $snap $dy
		} "t1Cursor" {
			set snap [expr $xPos - $t1Pos]
			set t1Pos $xPos
			.display move $cursorTag $snap $dy
			calculateCursors
		} "t2Cursor" {
			set snap [expr $xPos - $t2Pos]
			set t2Pos $xPos
			.display move $cursorTag $snap $dy
			calculateCursors
		}
		
	}
	
	set xStart $xPos
}

#Calculate Cursor Displays
#---------------------------
#This procedure performs the necessary calculation to update the
#cursor displays based on the current screen position of the cursors.
proc calculateCursors {} {
	global frequency \
		t1Pos \
		t2Pos \
		va1Pos \
		va2Pos \
		vb1Pos \
		vb2Pos \
		xDiv10 \
		horizontalBox \
		deltaT \
		deltaVA \
		deltaVB \
		verticalBoxA \
		verticalBoxB \
		yDiv10	\
		triggerMode	\
		leftPos	\
		rightPos	\
		sampleRates	\
		sampleRate

	#Caculate the difference between the cursor positions
	set diff [expr abs($t1Pos-$t2Pos)]
	set diff [format "%.9f" $diff]
	
	#Calculate the time between cursors
	if { ($triggerMode != "Hi-Res") } {
		set timeDiff [expr ($diff/$xDiv10)*$horizontalBox]
	} else {
		#Find the starting sample from the left cursor position
		set startSample [expr {$leftPos/512.0*32768}]
		puts "Starting sample $startSample"
		#Find the end sample from the right sample position
		set endSample [expr {$rightPos/512.0*32768}]
		puts "End sample $endSample"
		
		set dataSpan [expr {$endSample-$startSample}]
		
		set sampleIndex [lsearch $sampleRates $sampleRate]
		puts "sampleIndex $sampleIndex"
		set sRate [expr {20.0E6/(pow(2,$sampleIndex))}]
		puts "sRate $sRate"
		
		set sSpan [expr {(1/$sRate)*$dataSpan}]
		puts "sSpan $sSpan"
		
		set timeDiff [expr {$diff/400.0*$sSpan}]
		puts "timeDiff $timeDiff"
		
	}
	
	if {$timeDiff > 0} {
		#Calculate frequency and format for display
		set frequency [expr 1/$timeDiff]
		if { $frequency > 1E6} {
			set frequency [format "%3.3f" [expr $frequency/1E6]]
			set freqEnd "MHz"
		} elseif { $frequency > 1E3} {
			set frequency [format "%3.3f" [expr $frequency/1E3]]
			set freqEnd "kHz"
		} else {
			set frequency [format "%3.3f" $frequency]
			set freqEnd " Hz"
		}
		set frequency "$frequency $freqEnd"
		#Calculate the time differense and format the display
		set deltaT $timeDiff
		#set deltaT [expr $diff/$xDiv10*$horizontalBox]
		if { $deltaT < 1E-6} {
			set deltaT [format "%3.1f" [expr ($deltaT)/1E-9]]
			set tEnd "ns"
		} elseif { $deltaT < 1E-3} {
			set deltaT [format "%3.3f" [expr $deltaT/1E-6]]
			set tEnd "us"
		} elseif { $deltaT < 1} {
			set deltaT [format "%3.3f" [expr $deltaT/1E-3]]
			set tEnd "ms"
		} else {
			set deltaT [format "%3.3f" $deltaT]
			set tEnd " s"
		}
		set deltaT "$deltaT $tEnd"
		
	} else {
		set frequency "?"
		set deltaT "0"
	}
	
	#Calculatations for Voltage Cursors
	set vaDiff [expr $va1Pos-$va2Pos]
	set vaDiff [format "%.3f" $vaDiff]
	set vaDiff [expr $vaDiff/$yDiv10*$verticalBoxA*(-1)]
	if { $vaDiff < 1} {
		set vaDiff [expr $vaDiff/1E-3]
		set vaDiff [format "%3.1f" $vaDiff]
		set vEnd "mV"
	} else {
		set vaDiff [format "%3.3f" $vaDiff]
		set vEnd " V"
	}
	set deltaVA "$vaDiff $vEnd"
	set vbDiff [expr $vb1Pos-$vb2Pos]
	set vbDiff [format "%.3f" $vbDiff]
	set vbDiff [expr $vbDiff/$yDiv10*$verticalBoxB*(-1)]
	if { $vbDiff < 1} {
		set vbDiff [expr $vbDiff/1E-3]
		set vbDiff [format "%3.1f" $vbDiff]
		set vEnd "mV"
	} else {
		set vbDiff [format "%3.3f" $vbDiff]
		set vEnd " V"
	}
	set deltaVB "$vbDiff $vEnd"
	
}

#======================================
#	Part II - Time Cursors Procedures
#======================================

#Move Time Cursor
#-------------------
#This procdure is called when the user drags the time offset cursor
#on the screen.  It updates the position of the cursor on the screen
#and invokes another procedure "adjustTimeOffset" which crunches
#the numbers and sets up the hardware to read back the appropriate
#data from the scope.
proc moveTime { xPos } {
	global timePos	\
		xPlotStart	\
		xPlotEnd	\
		xStart	\
		cursorBorder	\
		xPlotWidth	\
		burstCount	\
		burstIncrement
	
	#Check to see if we have gone off of the screen
	if { $xPos > $xPlotEnd} {set xPos $xPlotEnd}
	if { $xPos < $xPlotStart} { set xPos $xPlotStart}

	#Move the cursor on the screen
	set dy 0
	set dx [expr $xPos - $xStart]
	set xStart $xPos
	set timePos $xPos
	.display move timePosCursor $dx $dy
	
	#Update the hardware
	adjustTimeOffset $timePos
}

#Move T1 Cursor
#----------------
#This procedure is called when the user drags the T1 cursor on the
#screen.  The procedure ensures that the user does not drag the 
#cursor off of the edge of the screen and updates the global variable
#which stores the current location of the cursor.
proc moveT1 { xPos } {
	global t1Pos	\
		xPlotStart	\
		xPlotEnd	\
		xStart	\
		cursorBorder	\
		xPlotWidth	\
	
	#Check to see if we have gone off of the screen
	if { $xPos > $xPlotEnd} {set xPos $xPlotEnd}
	if { $xPos < $xPlotStart} { set xPos $xPlotStart}

	#Move the cursor on the screen
	set dy 0
	set dx [expr $xPos - $xStart]
	set xStart $xPos
	set t1Pos $xPos
	.display move t1Cursor $dx $dy
	
	calculateCursors
}

#Move T2 Cursor
#----------------
#This procedure is called when the user drags the T2 cursor on the
#screen.  The procedure ensures that the user does not drag the 
#cursor off of the edge of the screen and updates the global variable
#which stores the current location of the cursor.
proc moveT2 { xPos } {
	global t2Pos	\
		xPlotStart	\
		xPlotEnd	\
		xStart	\
		cursorBorder	\
		xPlotWidth	\
	
	#Check to see if we have gone off of the screen
	if { $xPos > $xPlotEnd} {set xPos $xPlotEnd}
	if { $xPos < $xPlotStart} { set xPos $xPlotStart}

	#Move the cursor on the screen
	set dy 0
	set dx [expr $xPos - $xStart]
	set xStart $xPos
	set t2Pos $xPos
	.display move t2Cursor $dx $dy
	
	calculateCursors
}

#Toggle Time Cursors
#------------------------
#This process is used to toggle the time cursors.  When the 
#cursor are enabled this procedure draws the cursor lines and
#handles on the screen, binds mouse clicks on the cursors to 
#handler procedures, and adds measurement labels to the GUI.
#When the cursors are disabled, this procedure deletes the cursors
#and their handles from the canvas and removes the measurement
#labels from the GUI.
proc toggleTimeCursors {} {
	global timeCursorsEnable \
		t1Pos \
		t2Pos \
		yPlotStart \
		yPlotEnd \
		voltageCursorsEnable
	
	if {$timeCursorsEnable} {
		.display create line	\
			$t1Pos $yPlotStart \
			$t1Pos $yPlotEnd \
			-tag t1Cursor \
			-fill brown
		.display create text \
			$t1Pos [expr $yPlotStart -5] \
			-fill brown \
			-tag t1Cursor \
			-text "T1"
		.display create line \
			$t2Pos $yPlotStart \
			$t2Pos $yPlotEnd \
			-tag t2Cursor \
			-fill brown
		.display create text \
			$t2Pos [expr $yPlotStart -5] \
			-fill brown \
			-tag t2Cursor \
			-text "T2"
		.display bind t1Cursor <Button-1> {markXStart t1Cursor %x}
		.display bind t1Cursor <B1-Motion> {moveT1 %x}
		.display bind t2Cursor <Button-1> {markXStart t2Cursor %x}
		.display bind t2Cursor <B1-Motion> {moveT2 %x}
		grid .measurements.cursor -row 1 -column 0
		grid .measurements.cursor.freq
	} else {
		#Disable the cursors
		.display delete t1Cursor
		.display delete t2Cursor
		if { !$timeCursorsEnable && !$voltageCursorsEnable} {
			grid remove .measurements.cursor
		} else {
			grid remove .measurements.cursor.freq
		}
	}
}

#Adjust Time Offset
#-------------------
#This procedure is used to calculate the new address offset for data
#being read back from the scope.  The user is able to move the time
#cursor to either edge of the screen effectively positioning the trigger
#point at any time offset on the display.   This offset is subtracted 
#from the address in the SRAM memory where the trigger occurs.
proc adjustTimeOffset {cursorPosition} {
	global	cursorBorder	\
			burstCount		\
			burstIncrement	\
			yPlotWidth

	#Figure out where the time cursor is, accounting for the border
	#around the edge of the screen
	set plotPosition [expr $cursorPosition-$cursorBorder]
	
	#Use the plotPosition calculated above to determine (in ratio form)
	#how far the cursor is from the left edge of the screen
	set triggerRatio [format "%.2f" [expr $plotPosition/($yPlotWidth*1.0)]]
	
	#The number of points returned from the hardware is given by
	#burstCount.  We apply the ratio calculated above to the 
	#burstCount to determine how many data points make up the
	#offset.
	set countOffset [expr round($triggerRatio * $burstCount)] 
	
	#We calculated the actual SRAM address offset (a 16 bit number)
	#by multiplying the number of points in the offset (countOffset)
	#by the interval between each point.
	set addressOffset [expr $burstIncrement * $countOffset]
	
	#Here we convert the addressOffset into to 8-bit integers and
	#send the values to the hardware
	set upperAddr [expr $addressOffset/256]
	set upperAddr [expr floor($upperAddr)]
	set upperAddr [expr round($upperAddr)]
	set lowerAddr [expr $addressOffset - ($upperAddr*256)]
	sendCommand "u$upperAddr"
	sendCommand "d$lowerAddr"
}

#======================================
#	Part III - Voltage Cursors Procedures
#======================================

#Move Voltage Cursor
#----------------------
#This procedure called when one of the voltage cursor is dragged
#on the screen by the user.  The procedure ensures that the user
#does not drag the cursor off the screen.
proc moveVcursor { vTag vPos } {
	global va1Pos \
		va2Pos \
		vb1Pos \
		vb2Pos \
		yPlotStart	\
		yPlotEnd	\
		yStart	\
		cursorBorder	\
		yPlotWidth
	
	#Check to see if we have gone off of the screen
	if { $vPos > $yPlotEnd} {set vPos $yPlotEnd}
	if { $vPos < $yPlotStart} { set vPos $yPlotStart}

	#Move the cursor on the screen
	set dx 0
	set dy [expr $vPos - $yStart]
	set yStart $vPos
	
	#Save the new position
	switch $vTag {
		"va1Cursor" {
			set va1Pos $vPos
		} "va2Cursor" {
			set va2Pos $vPos
		} "vb1Cursor" {
			set vb1Pos $vPos
		} "vb2Cursor" {
			set vb2Pos $vPos
		}
	}
	
	#Move the cursor line on the screen
	.display move $vTag $dx $dy
	
	#Update cursor label displays
	calculateCursors
}

#Toggle Voltage Cursors
#------------------------
#This process is used to toggle the voltage cursors.  When the 
#cursor are enabled this procedure draws the cursor lines and
#handles on the screen, binds mouse clicks on the cursors to 
#handler procedures, and adds measurement labels to the GUI.
#When the cursors are disabled, this procedure deletes the cursors
#and their handles from the canvas and removes the measurement
#labels from the GUI.
proc toggleVoltageCursors {} {
	global voltageCursorsEnable \
		timeCursorsEnable \
		va1Pos \
		va2Pos \
		vb1Pos \
		vb2Pos \
		xPlotStart \
		xPlotEnd 
	
	if {$voltageCursorsEnable} {
		#Cursors for Channel A
		.display create line	\
			$xPlotStart $va1Pos \
			$xPlotEnd $va1Pos \
			-tag va1Cursor \
			-fill red \
			-dash -
		.display create text \
			[expr $xPlotStart + 15] [expr $va1Pos - 10]\
			-fill red \
			-tag va1Cursor \
			-text "VA1"
		.display create line \
			$xPlotStart $va2Pos\
			$xPlotEnd $va2Pos\
			-tag va2Cursor \
			-fill red \
			-dash -
		.display create text \
			[expr $xPlotStart +15] [expr $va2Pos -10] \
			-fill red \
			-tag va2Cursor \
			-text "VA2"
		#Cursors for Channel B
		.display create line	\
			$xPlotStart $vb1Pos \
			$xPlotEnd $vb1Pos \
			-tag vb1Cursor \
			-fill blue \
			-dash -
		.display create text \
			[expr $xPlotEnd - 15] [expr $vb1Pos-10]\
			-fill blue \
			-tag vb1Cursor \
			-text "VB1"
		.display create line \
			$xPlotStart $vb2Pos\
			$xPlotEnd $vb2Pos\
			-tag vb2Cursor \
			-fill blue \
			-dash -
		.display create text \
			[expr $xPlotEnd - 15] [expr $vb2Pos-10]\
			-fill blue \
			-tag vb2Cursor \
			-text "VB2"
		
		#Bind Mouse Clicks to the various cursors
		.display bind va1Cursor <Button-1> {markYStart va1Cursor %y}
		.display bind va1Cursor <B1-Motion> {moveVcursor va1Cursor %y}
		.display bind va2Cursor <Button-1> {markYStart va2Cursor %y}
		.display bind va2Cursor <B1-Motion> {moveVcursor va2Cursor %y}
		
		.display bind vb1Cursor <Button-1> {markYStart vb1Cursor %y}
		.display bind vb1Cursor <B1-Motion> {moveVcursor vb1Cursor %y}
		.display bind vb2Cursor <Button-1> {markYStart vb2Cursor %y}
		.display bind vb2Cursor <B1-Motion> {moveVcursor vb2Cursor %y}
		
		grid .measurements.cursor -row 1 -column 0
		grid .measurements.cursor.voltage
	} else {
		#Remove the cursors and cursor display labels
		.display delete va1Cursor
		.display delete va2Cursor
		.display delete vb1Cursor
		.display delete vb2Cursor
		if { !$timeCursorsEnable && !$voltageCursorsEnable} {
			grid remove .measurements.cursor
		} else {
			grid remove .measurements.cursor.voltage
		}
	}
}

#======================================
#	Part IV - Ground Reference Cursors Procedures
#======================================

#Move Channel A Ground Reference
#------------------------------------
#This procedure is called when the user grabs the channel reference
#cursor and drags it up or down on the screen.  This procedure moves
#the cursor, ensures that the user doesn't drag the cursor off the 
#screen, and updates the waveform offset for plotting purposes.
proc moveChAGnd { yPos } {
	global chAGndPos	\
		yStart	\
		yPlotStart	\
		yPlotEnd	\
		triggerSource	\
		waveOffsetA \
		trigPos
	
	#Make sure we haven't gone off the screen
	if { $yPos > $yPlotEnd } { set yPos $yPlotEnd }
	if { $yPos < $yPlotStart } { set yPos $yPlotStart }
	
	#Move the cursor
	set dx 0
	set dy [expr $yPos - $yStart]
	set yStart $yPos
	set chAGndPos $yPos
	.display move chAGnd $dx $dy
	
	#Move the trigger cursor if trigger source is A
	if { $triggerSource == "A" } {
		.display move trigLevelCursor $dx $dy
		set trigPos [expr $trigPos + $dy]
	}
	
	#Calculate the waveform screen offset
	set waveOffsetA [expr $chAGndPos-(($yPlotEnd-$yPlotStart)/2)]
	
	
}

#Move Channel B Ground Reference
#------------------------------------
#This procedure is called when the user grabs the channel reference
#cursor and drags it up or down on the screen.  This procedure moves
#the cursor, ensures that the user doesn't drag the cursor off the 
#screen, and updates the waveform offset for plotting purposes.
proc moveChBGnd { yPos } {
	global chBGndPos	\
		yStart		\
		yPlotStart		\
		yPlotEnd	\
		triggerSource	\
		waveOffsetB \
		trigPos
	
	#Make sure we haven't gone off the screen
	if { $yPos > $yPlotEnd } { set yPos $yPlotEnd }
	if { $yPos < $yPlotStart } { set yPos $yPlotStart }

	#Move the cursor
	set dx 0
	set dy [expr $yPos - $yStart]
	set yStart $yPos
	set chBGndPos $yPos
	.display move chBGnd $dx $dy
	
	#Move the trigger cursor too, if the trigger source is B
	if { $triggerSource == "B" } {
		.display move trigLevelCursor $dx $dy
		set trigPos [expr $trigPos + $dy]
	}
	
	#Calculate the waveform screen offset
	set waveOffsetB [expr $chBGndPos-(($yPlotEnd-$yPlotStart)/2)]
}

#======================================
#	Part V - Trigger Cursor Procedures
#======================================

#Toggle Trigger Source
#----------------------
#This procedure is called when the user changes the trigger source.
#The procedure snaps the trigger cursor to the new reference
#position.
proc toggleSource {} {
	global triggerSource	\
		chAGndPos	\
		chBGndPos	
		
	set dx 0
	if { $triggerSource == "A" } {
		set dy [expr $chAGndPos - $chBGndPos]
	} else {
		set dy [expr $chBGndPos - $chAGndPos]
	}
	.display move trigLevelCursor $dx $dy
}

#Move Trigger
#--------------
#This procedure is called when the user drags the trigger cursor
#on the screen.  It calculates the trigger voltage corresponding
#to the current screen position and sends the appropriate command
#to the hardware.
proc moveTrigger { yPos } {
	global trigPos	\
		yStart	\
		yPlotEnd	\
		yPlotStart	\
		yPlotWidth	\
		triggerLevel	\
		triggerVoltage	\
		cursorBorder	\
		chAGndPos	\
		chBGndPos	\
		triggerSource	\
		verticalBoxA	\
		verticalBoxB	\
		stepA	\
		stepB
	
	#Determine whether the trigger source is A or B
	if { $triggerSource == "A" } {
		set reference $chAGndPos
		set boxSize $verticalBoxA
		set stepSize $stepA
	} else {
		set reference $chBGndPos
		set boxSize $verticalBoxB
		set stepSize $stepB
	}
	
	#Check to see if we have gone off of the screen
	if { $yPos > $yPlotEnd } { set yPos $yPlotEnd }
	if { $yPos < $yPlotStart } { set yPos $yPlotStart }
	
	#Calculate the difference between the trigger cursor
	#and its reference in A/D units
	set difference [expr $yPos-$reference]
	set numDiv [expr $difference/40.0]
	set absDiv [expr abs($numDiv)]
	
	#Check to see if we have exceeded the maximum or
	#minimum trigger level
	if { $absDiv > 5.0 } {
		if { $numDiv < 0 } {
			set numDiv {-5.0}
		} else {
			set numDiv {5.0}
		}
		set yPos [expr $reference + ($numDiv*40.0)]
	}

	#Move the cursor on the screen
	set dx 0
	set dy [expr $yPos - $yStart]
	
	set yStart $yPos
	set trigPos $yPos
	.display move trigLevelCursor $dx $dy
	
	#Calculate the trigger voltage that corresponds to the
	#current trigger cursor position
	set triggerVoltage [expr $numDiv*$boxSize]
	
	#Calculate the trigger value to send to the hardware
	set triggerSteps [expr $triggerVoltage/$stepSize]
	set triggerSteps [expr int($triggerSteps)]
	set triggerLevel [expr 128+$triggerSteps]

	#Format the trigger voltage for the display
	#(The value must be multiplied by -1)
	set triggerVoltage [expr $triggerVoltage*-1.0]
	set triggerVoltage [format "%.3f" $triggerVoltage]
	set triggerVoltage "$triggerVoltage V"
	
	#Update the hardware
	 sendCommand "t$triggerLevel" 
	
}

proc drawXCursor {} {
	global timePos	\
		yPlotStart	\
		yPlotEnd

	#Place the Time Scroll Cursor
	.display create line	\
		$timePos $yPlotStart	\
		$timePos $yPlotEnd	\
		-tag timePosCursor	\
		-fill violet
	.display create text	\
		$timePos [expr $yPlotStart - 5] \
		-fill violet	\
		-tag timePosCursor	\
		-text "X"

}
