#File: hi-res.tcl
#Syscomp USB Oscilloscope GUI
#Hi-Resolution Browser Procedures and Widgets

#JG
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#Toggle Hi-Res Window
#-------------------------
#This procedure toggles the Hi-Res browser window.  It removes the X grid marks
#when switching to hi-res mode and puts them back when switching out of hi-res mode
proc toggleHRWindow {mode} {
	global hrEnable	\
		xPlotStart	\
		xPlotEnd	\
		yPlotStart	\
		yPlotEnd	\
		xDiv10
	
	if {$mode == "off"} {
		wm withdraw .hr
		set hrEnable 0
		#Redraw the horizontal grid markers
		for {set xpos $xPlotStart} { $xpos < $xPlotEnd } { incr xpos $xDiv10} {
			.display create line $xpos $yPlotStart $xpos $yPlotEnd -fill black -dash . -tag  vertGrid
		}
	} elseif { $mode == "on"} {
		wm deiconify .hr
		wm protocol .hr WM_DELETE_WINDOW {tk_messageBox -title "Info" -default ok -message "Use the Trigger Mode Control in the Main Window to Close This Window" -type ok -icon info}
		set hrEnable 1
		.display delete vertGrid
	}
}

#Mark Hi-Res Cursor Start Position
#---------------------------------------
#This procedure is called when the use clicks on one of the two hi-res cursors.  It
#stores the starting position of the cursor in a global variable and ensures that the
#clicked cursor snaps to the mouse pointer position.
proc markHRStart {cursorTag xPos} {
	global leftPos	\
		rightPos	\
		hrStart	\
		hrDist	\
		hrLock
	
	if {$hrLock} {
		set hrDist [expr {$rightPos - $leftPos}]
	}

	switch $cursorTag {
		"leftCursor" {
			set snap [expr $xPos - $leftPos]
			set leftPos $xPos
			.hr.browser move $cursorTag $snap 0
			if { $hrLock } {
				set rightPos [expr {$rightPos + $snap}]
				.hr.browser move rightCursor $snap 0
			}
		} "rightCursor" {
			set snap [expr $xPos - $rightPos]
			set rightPos $xPos
			.hr.browser move $cursorTag $snap 0
			if { $hrLock } {
				set leftPos [expr {$leftPos + $snap}]
				.hr.browser move leftCursor $snap 0
			}
		}
	}

	set hrStart $xPos
	
	
	
}

#Move Left Cursor
#--------------------
#This procedure is called when the user drags the left hi-res cursor.  It updates the
#position of the cursor on the canvas and calls the updateScopeDisplay to redraw the
#channel A and B traces on the main scope display based on the new cursor position.
proc moveLeftCursor { xPos } {
	global leftPos	\
		rightPos	\
		hrStart	\
		hrLock	\
		hrDist

	#Check to see if we have gone off the screen
	if { $xPos < 0} {set xPos 0}
	#Make sure we don't overrun the right cursor
	if { $xPos >= $rightPos} {set xPos [expr {$rightPos-3}]}
	
	if {$hrLock} {
		if { [expr {$xPos+$hrDist}] > 512} {
			set xPos [expr {512 - $hrDist}]
			puts "$xPos"
		}
	}
	
	#Move the cursor on the screen
	set dx [expr {$xPos - $hrStart}]
	set hrStart $xPos
	set leftPos $xPos
	.hr.browser move leftCursor $dx 0

	if {$hrLock} {
		set rightPos [expr {$xPos + $hrDist}]
		.hr.browser move rightCursor $dx 0
	}

	updateScopeDisplay
}

#Move Right Cursor
#---------------------
#This procedure is called when the user drags the right hi-res cursor.  It updates the
#position of the cursor on the canvas and calls the updateScopeDisplay to redraw the
#channel A and B traces on the main scope display based on the new cursor position.
proc moveRightCursor { xPos } {
	global leftPos	\
		rightPos	\
		hrStart	\
		hrLock	\
		hrDist
		
	#Check to see if we have gone off the screen
	if { $xPos > 511} {set xPos 511}
	#Make sure we don't overrun the left sensor
	if { $xPos <= $leftPos} {set xPos [expr {$leftPos + 3}]}
	
	if {$hrLock} {
		set temp [expr {$xPos - $hrDist}]
		puts "$xPos $hrDist $temp"
		if { $temp < 0} {
			set xPos $hrDist
		}
	}
	
	#Move the cursor on the screen
	set dx [expr { $xPos - $hrStart}]
	set hrStart $xPos
	set rightPos $xPos
	.hr.browser move rightCursor $dx 0
	
	if {$hrLock} {
		set leftPos [expr {$xPos - $hrDist}]
		.hr.browser move leftCursor $dx 0
	}

	updateScopeDisplay
}


#Update Scope Display
#-------------------------
#This procedure is called to plot channel A and B traces on the main scope display
#based on the position of the cursors in the hi-res browser.  The routine calculates
#the samples which correspond to the screen positions of the "Start" and "End" 
#cursors in the hi-res browsers.  After calculating the starting and ending samples,
#the procedure extracts that data range from the captured data and plots it on the
#main scope display.
proc updateScopeDisplay {} {
	global leftPos	\
		rightPos	\
		dataA	\
		dataB	\
		waveColorA	\
		waveColorB	\
		waveOffsetA	\
		waveOffsetB	\
		waveTagA	\
		waveTagB	\
		xPlotStart	\
		mathEnabled	\
		dataARaw	\
		dataBRaw	\
		exportData	\
		stepA	\
		stepB		\
		hrSamplesSelected	\
		xyMode	\
		fftEnable	\
		histoEnabled
	
	#Make sure that we have data to look at.
	if { ([llength $dataA] < 32768) || ([llength $dataB] < 32768) } {
		return
	}
	
	#Find the starting sample from the left cursor position
	set startSample [expr {round($leftPos/512.0*32768)}]
	#Find the end sample from the right sample position
	set endSample [expr {round($rightPos/512.0*32768)}]
	
	#Plot data on the oscilloscope display
	set dataSpan [expr {$endSample-$startSample}]
	set plotDataA {}
	set plotDataB {}
	set xData {}
	for {set i 0} {$i <= $dataSpan } {incr i} {
		set temp [expr {$xPlotStart+($i*1.0/$dataSpan)*400}]
		lappend plotDataA $temp
		lappend plotDataB $temp
		lappend xData $temp
		lappend plotDataA [lindex $dataA [expr {$i+$startSample}]]
		lappend plotDataB [lindex $dataB [expr {$i+$startSample}]]
	}
	
	.display delete $waveTagA
	.display delete $waveTagB
	.display create line	\
		$plotDataA	\
		-tag $waveTagA	\
		-fill $waveColorA
	.display create line	\
		$plotDataB	\
		-tag $waveTagB	\
		-fill $waveColorB
		
	#Save values for export
	set exportData {}
	lappend exportData $xData
	lappend exportData [lrange $dataARaw $startSample $endSample]
	lappend exportData [lrange $dataBRaw $startSample $endSample]
	#Calculate the sample rate
	set temp [expr {20.0E6/(pow(2,[sampleConstant]))}]
	#Effective "horizontalBox"
	lappend exportData [expr {($dataSpan/(10.0*$temp))}]
	lappend exportData $stepA
	lappend exportData $stepB
	
	#Update the scope display cursors
	calculateCursors
	
	if {$histoEnabled} {updateHisto}
	
	if $mathEnabled {
		updateMath
	}
	
	if $xyMode {
		plotXY [lrange $dataARaw $startSample $endSample] \
			[lrange $dataBRaw $startSample $endSample]
	}
	
	if $fftEnable {updateFFT}
	
	set hrSamplesSelected $dataSpan
}

#Plot Browse
#-------------
#This procedure plots a representation of the entire 64k (2x32k) samples
#captured by the scope on the hi-res browser display.
proc plotBrowse {yData waveColor waveTag} {
	global burstCount
	
	.hr.browser delete $waveTag
	
	set plotData {}
	set i 0
	while { $i < $burstCount} {
		lappend plotData [expr {($i/1.0)/$burstCount*512}]
		set y [lindex $yData $i]
		lappend plotData [expr {($y-128)/256.0*32+64}]
		#incr i
		set i [expr {$i + 1}]
	}
	
	.hr.browser create line	\
		$plotData	\
		-tag $waveTag	\
		-fill $waveColor
	
	updateScopeDisplay
}

#Widgets for the Hi-Res Browser
#-------------------------------------

toplevel .hr
wm resizable .hr 0 0
wm title .hr "Hi-Res Browser"

label .hr.title	\
	-text "Hi-Resolution Browser"	\
	-font {-weight bold -size 12}

canvas .hr.browser	\
	-width 512		\
	-height 128		\
	-background white

#Draw the x-axis
.hr.browser create line 0 64 512 64 -fill black
#Draw the trigger point
.hr.browser create line 256 96 256 32 -fill violet
.hr.browser create text 256 18 -text Trigger -fill violet
.hr.browser create text 256 28 -text Point -fill violet
#Draw the browser cursors
.hr.browser create line 253 96 253 35 -fill red -tag leftCursor
.hr.browser create oval 250 35 256 40 -fill red -tag leftCursor
.hr.browser create text 253 100 -text Start -fill red -tag leftCursor
.hr.browser create line 259 96 259 35 -fill blue -tag rightCursor
.hr.browser create oval 256 35 262 40 -fill blue -tag rightCursor
.hr.browser create text 259 110 -text End -fill blue -tag rightCursor
#.hr.browser create line 253 35 259 35 -fill orange -tag hrCursor
#.hr.browser create line 256 35 256 30 -fill orange -tag hrCursor
#.hr.browser create polygon 256 30 250 24 262 24 -fill orange -tag hrCursor

set leftPos 253
set rightPos 259

.hr.browser bind leftCursor <Button-1> {markHRStart leftCursor %x}
.hr.browser bind leftCursor <B1-Motion> {moveLeftCursor %x}

.hr.browser bind rightCursor <Button-1> {markHRStart rightCursor %x}
.hr.browser bind rightCursor <B1-Motion> {moveRightCursor %x}

#.hr.browser bind hrCursor <Button-1> { markHRStart %x}
#.hr.browser bind hrCursor <B1-Motion> {moveHR %x}

package require BWidget
ProgressBar .hr.progress	\
	-maximum 100	\
	-variable hrProgress	\
	-orient horizontal 	\
	-width 100 	\
	-height 20

checkbutton .hr.lock	\
	-text "Lock Cursor Spacing"	\
	-variable hrLock
	
frame .hr.info	\
	-relief groove	\
	-borderwidth 2
	
label .hr.info.sampleLabel	\
	-text "Number of Samples Selected:"
	
label .hr.info.selectedSamples	\
	-relief sunken	\
	-textvariable hrSamplesSelected	\
	-width 6
	
grid .hr.info.sampleLabel -row 0 -column 0
grid .hr.info.selectedSamples -row 0 -column 1

grid .hr.title -row 0 -column 0 -sticky w
grid .hr.browser -row 1 -column 0
grid .hr.progress -row 2 -column 0
grid .hr.lock -row 3 -column 0
grid .hr.info -row 4 -column 0
#pack .hr.browser
#pack .hr.progress
#pack .hr.lock

#Remove the Hi-Res Browser until Hi-Res Mode is selected
toggleHRWindow off