#File: usb_serial.tcl
#Syscomp USB Oscilloscope GUI
#USB Serial Port Interface Procedures

#JG
#File Version 1.03 January 2006

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#======================================

#Open USB Serial Port
#----------------------
# This procedure opens the serial port and configures it to
# the appropriate baud rate.  The procedure returns 0 if there
# was an error while trying to open the port and 1 if it was
# successful.  Once opened, the port can be accessed through
# the global variable "portHandle".  This procedure also sets up
#the fileevent handler which handles incoming data from the scope.
proc openSerialPort {} {
	global 	portHandle	\
			serialPort	\
			serialStatus	\
			baudRate	\
			serialCheck	\
			connectMessage	\
			osType

	
	if { [catch {set portHandle [open $serialPort r+]} result] } {
		set answer [tk_messageBox	\
			-title "Hardware Error"	\
			-default no	\
			-message $connectMessage	\
			-type	yesno	\
			-icon warning]
		puts "Unable to connect to oscilloscope!"
		set serialStatus "Disconnected"
		.menubar.serialPortStatus configure \
			-background red
		#Send all commands to the console
		set portHandle stdout
		#Initialize Values so the GUI is function in disconnect mode
		initializeScope
		
		if {$answer == "yes"} {
			showSerialSettings
		}
		
		return 0
	} else {
		#The scope defaults to 38kbps on power up, so we will
		#try talking to the scope at 38kbps first.
		if {$osType!="Darwin"} {
			fconfigure $portHandle \
				-mode $baudRate,n,8,1	\
				-blocking 0			\
				-buffering line 		\
				-handshake rtscts 
		} else {
			exec stty -f $serialPort $baudRate cs8 -parenb -cstopb
			fconfigure $portHandle	\
				-blocking 0		\
				-buffering line
		}
				
		#We are now going to query the device to make
		#sure it is a scope.  We set up  and intermediate 
		#fileevent handler to deal with identification data 
		#received from the scope.
		fileevent $portHandle readable {
			set incomingData [gets $portHandle]
			puts "incomingData: $incomingData"
			if {  [lsearch $incomingData "Oscilloscope"] !=-1 } {
				set serialCheck found
				set firmwareIdent $incomingData
			} else {
				puts "No match"
			}
		}		
		puts "Trying 38.4kbps..."
		#Query the device.
		sendCommand ""
		sendCommand ""
		sendCommand i
		#Wait for a response from the device
		set serialCheck waiting
		after 1500 {set serialCheck timeout}
		vwait serialCheck
		
		if { $serialCheck == "found" } {
			puts "Connected."
			#Display "Connected" status on menu bar
			set serialStatus "Connected"
			.menubar.serialPortStatus configure -background green
			
			#Change to the fastest baud (250kbps)
			set baudRate 230400
			sendCommand b1
			if {$osType!="Darwin"} {
				fconfigure $portHandle 	\
					-mode $baudRate,n,8,1 \
					-blocking 0 		\
					-buffering line 		\
					-handshake rtscts	\
					-encoding binary	\
					-translation {binary auto}
			} else {
				exec stty -f $serialPort $baudRate cs8 -parenb -cstopb
				fconfigure $portHandle	\
					-blocking 0		\
					-buffering line	\
					-encoding binary	\
					-translation {binary auto}
			}
			
			#Set up the real fileevent handler to deal with
			#data received from the scope
			setupFileevent

			#Send a few newlines to flush out the buffers and
			#align the UARTs
			sendCommand "i"
			sendCommand "i"
			sendCommand "i"
		
			#Change to Binary Transfer Mode
			sendCommand "B"
		
			#Initialize the hardware with current settings
			initializeScope
	
			#Start the scope
			selectChannel
			return 1
		} else {
			puts "Failed"
			#The scope may already be set to the higher
			#baud rate.  Try again at 230kbps.
			set baudRate 230400
			if {$osType!="Darwin"} {
				fconfigure $portHandle 	\
					-mode $baudRate,n,8,1 \
					-blocking 0 		\
					-buffering line 		\
					-handshake rtscts
			} else {
				exec stty -f $serialPort $baudRate cs8 -parenb -cstopb
				fconfigure $portHandle	\
					-blocking 0		\
					-buffering line
			}
			
			#Flush the buffers, align UARTs
			sendCommand ""
			sendCommand ""
			
			puts "Trying 230kbps..."
			#Query the device.
			sendCommand i
			#Wait for a response from the device
			set serialCheck waiting
			after 1500 {set serialCheck timeout}
			vwait serialCheck
		
			if {$serialCheck == "found"} {
				puts "Connected."
				#Display "Connected" status on menu bar
				set serialStatus "Connected"
				.menubar.serialPortStatus configure \
					-background green
				
				#Set up to receive binary data from the scope
				fconfigure $portHandle		\
					-encoding binary	\
					-translation {binary auto}
				#Change to Binary Transfer Mode
				sendCommand "B"
				
				setupFileevent
				
				#Initialize the hardware with current settings
				initializeScope
	
				#Start the scope
				selectChannel
				return 1
			} else {
				puts "Failed"
				#Display "Disconnected" status on the menubar
				set serialStatus "Disconnected"
				.menubar.serialPortStatus configure \
					-background red
				tk_messageBox	\
					-title "Hardware Error"	\
					-default ok		\
					-message "Unknown Device on $serialPort"	\
					-type ok			\
					-icon warning
				catch { [close $portHandle]}
			}
		}
	}		
}

#Set Up Fileevent
#-----------------
#This procedure is called to initialize the fileevent handler
#for data received from the scope.  The handler receives the data
#in binary form as signed integers, and then converts it
#to an unsigned integer, before calling the updateOscilloscope
#procedure which processes the data and displays it on the screen.
proc setupFileevent {} {
	global portHandle	\
		dataCount	\
		signed
	
	#Read in any left over data (such as line ends) from the
	#autodetection routines.
	set junk [read $portHandle]
	set dataCount 0
	set unsigned {}

	fileevent $portHandle readable {
		set incomingData [read $portHandle]
		binary scan $incomingData c* signed
		foreach byte $signed {
			lappend unsigned [lindex $cvt [expr {$byte & 255}]]
			incr dataCount
		}
		
		#puts "Processing data..."
		
		#Removed due to File I/O Errors
		#Moved to updateOsilloscope (after all data has been received)
		#processData $signed
		
		if {$dataCount == [expr 2*$burstCount]} {
			#We have received the correct amount of data
			#from the scope.  Process and display it.
			if { $triggerMode == "Hi-Res" } {
				.hr.browser delete downloading
			}
			updateOscilloscope $unsigned
			
			#MOVED TO acquireWaveform
			#set dataCount 0
			#set unsigned {}
		} elseif { $dataCount > [expr 2*$burstCount]} {
			#This is bad.  We have received too much data
			#from the scope.  Hopefully, the PC and scope are
			#simply out of alignment.  We read in all of the data,
			#discard it, and try again.
			puts "$dataCount GARBAGE in incoming buffer"
			set garbage [read $portHandle]
			set dataCount 0
			set unsigned {}
			acquireWaveform
		} elseif { ($dataCount < [expr {2*$burstCount}]) && ($triggerMode == "Hi-Res")} {
			catch {after cancel $downloadTimeout}
			.hr.browser delete downloading
			set percentDone [expr {round($dataCount/65536.0*100)}]
			puts "percentDone $percentDone"
			set hrProgress $percentDone
			.hr.browser create text 256 8 -text "DOWNLOADING DATA...$percentDone %" -tag downloading
			set downloadTimeout [after 1000 {.hr.browser delete downloading}]
		}	
	}
}

proc processData {signed} {
	global cvt	\
		dataARaw	\
		dataA	\
		dataBRaw	\
		dataB	\
		dataCount	\
		waveOffsetA	\
		waveOffsetB	\
		invertA	\
		invertB

	foreach byte $signed {
		set datum [lindex $cvt [expr {$byte & 255}]]
		
		#Sort the incoming data into each of the lists.  The data is
		#interleaved.  i.e. A0B0A1B1A2B2A3B3A4B4A5B5.
		#We also convert the data into screen coordinates while
		#we're at it.
		if { [llength $dataARaw] == [llength $dataBRaw] } {
			if $invertA {
				set datum [expr {(-1)*($datum-255)}]
			}
			lappend dataARaw $datum
			lappend dataA [datatoscreeny $datum $waveOffsetA a]
		} else {
			if $invertB {
				set datum [expr {(-1)*($datum-255)}]
			}
			lappend dataBRaw $datum
			lappend dataB [datatoscreeny $datum $waveOffsetB b]
		}
			
		incr dataCount
	}
}

#Close Serial Port
#-----------------
#This procedure closes any USB serial port currently open as
#"portHandle". 
proc closeSerialPort {} {
	global portHandle	\
		serialStatus	
	
	if {$portHandle != "stdout"} {
		catch { [close $portHandle]}
	}
	set serialStatus "Disconnected"
	.menubar.serialPortStatus configure -background red
	set portHandle {}
}

#Autodetect
#------------
#This procedure will detect the first available USB scope connected
#to the computer.  This is accomplished by attempting to open COM
#ports 1 through 9, in order.  If a COM port is successfully opened,
#the device on the com port is queried.  A syscomp oscilloscope will
#respond to this query with a descriptor string.
proc autodetect {} {
	global baudSlow	\
		portHandle	\
		serialCheck	\
		serialCheckHandle
	
	#Close the serial port in case the user has already
	#connected to the scope.
	closeSerialPort
	
	#Ask the user to unplug and then reconnect the scope.  This
	#will reset the scope to its initial baud rate of 38400.
	tk_messageBox	\
		-title "Unplug..."	\
		-default ok		\
		-message "Please Disconnect and then Reconnect the Scope"	\
		-type ok			\
		-icon info
	
	text .serial.autoStatus	\
		-height 10		\
		-yscrollcommand ".serial.textScroll set"
	scrollbar .serial.textScroll	\
		-orient vert	\
		-command ".serial.autoStatus yview"
	.serial.autoStatus insert end "Beginning autodetection...\n"
	
	button .serial.autoBack	\
		-text "<< Back"		\
		-command {
			destroy .serial.autoStatus
			destroy .serial.autoBack
			destroy .serial.textScroll
			grid .serial.ports -row 1 -column 0
			grid .serial.back -row 2 -column 0
		}
		
	#Replace the serial port radio buttons with the text widget
	grid remove .serial.ports
	grid remove .serial.back
	grid .serial.autoStatus -row 1 -column 0
	grid .serial.textScroll -row 1 -column 1 -sticky ns
	grid .serial.autoBack -row 2 -column 0 -columnspan 2
	
	#Disable the autodetect button in case the user gets bored and
	#starts pressing buttons.
	.serial.autoBack configure -state disabled

	#Here we loop through COM ports 1-99, attempting to open them
	for {set i 3} {$i <100} {incr i} {
		if {$i<10} {
			set serialPort "com$i:"
		} else {
			set serialPort "\\\\.\\com$i"
		}
		if { [catch {set portHandle [open $serialPort r+]} result] } {
			puts "Unable to open $serialPort: $result"
			flush stdout
			.serial.autoStatus insert end "COM$i: Unavailable\n"
			update
		} else {
			#We were able to open a port.  Configure baud
			#settings and prepare to query the device.
			fconfigure $portHandle		\
				-mode $baudSlow,n,8,1	\
				-blocking 1			\
				-buffering line		\
				-handshake rtscts
			
			#Here we set up an intermediate fileevent handler
			#to receive descriptor strings from the scope.
			fileevent $portHandle readable {
				set incomingData [gets $portHandle]
				puts "incomingData: $incomingData"
				if {  [lsearch $incomingData "Oscilloscope"] !=-1 } {
					set serialCheck found
				} else {
					puts "No match"
				}
			}

			#Query the device.
			puts $portHandle "i"
			puts "$serialPort--> i"
			.serial.autoStatus insert end "COM$i: Opened..."
			flush stdout
			update
			
			#Set up a timeout in case the device on this COM
			#port does not repsond to the query.
			set serialCheck waiting
			.serial.autoStatus insert end "Querying..."
			set serialCheckHandle [after 1500 {set serialCheck timeout}]
			vwait serialCheck
			
			if { $serialCheck == "found" } {
				#Device responded on this COM port.  We've
				#located the scope.
				tk_messageBox	\
					-title "Hardware Found"	\
					-default ok		\
					-message "DSO-101 Found on COM$i"	\
					-type ok			\
					-icon info	\
					-parent .serial
				puts "Oscilloscope found on $serialPort"
				raise .serial
				catch { [close $portHandle]}
				.serial.autoBack configure -state normal
				return $serialPort
			} else {
				puts "No Device on $serialPort"
				.serial.autoStatus insert end "Not a Syscomp Device\n"
				catch { [close $portHandle]}
			}
		}
	}
	#At this point, we've been through all 9 COM ports without any
	#luck.  We relay the bad news to the user.
	tk_messageBox	\
			-title "Hardware Error"	\
			-default ok		\
			-message "Unable to Autodetect"	\
			-type ok			\
			-icon warning	\
			-parent .serial
	.serial.autoBack configure -state normal
	
	#Autodetection failed
	return ""


}

#Send Command to Hardware
#------------------------------
# This procedure takes the argument "command" and sends it
# to the serial port.  The argument is also printed to stdout.
proc sendCommand {command} {
	global portHandle	\
		osType
	
	puts $portHandle $command
	if {$osType == "Darwin"} {flush $portHandle}
	puts $command
}

#Serial Settings
#---------------
#When invoked, this procedure creates a new window which allows
#the user to adjust the serial port settings.  The settings are read 
#from, and stored to, a file called scopeport.cfg.
proc showSerialSettings {} {
	global osType
	
	closeSerialPort

	#Create a new window
	toplevel .serial
	wm title .serial "Port Settings"
	wm iconname .serial "Serial Settings"
	grab .serial
	focus .serial
	
	#Center the serial port settings window over the parent
	#See http://wiki.tcl.tk/534
	set w [winfo width .]
	set h [winfo height .]
	set x [winfo rootx .]
	set y [winfo rooty .]
	set xpos "+[ expr {$x+($w-[winfo width .serial])/2}]"
	set ypos "+[ expr {$y+($h-[winfo height .serial])/2}]"
	wm geometry .serial "$xpos$ypos"
	
	frame .serial.ports 	\
		-relief sunken	\
		-borderwidth 2
		
	label .serial.serialTitle	\
		-text "Port Settings"	\
		-font {-weight bold}
		
	#Create the radio buttons which allow the user to select the COM port.
	#COM ports have different names in the different operating systems, so
	#this procedure is different for each operating system
	set validPorts {}
	
	if {$osType == "windows"} {
		
		#Create a list of valid port names
		for {set i 0} {$i < 10} {incr i} {
			lappend validPorts "com$i"
		}
		
		#Create radio buttons for each of the COM ports
		for {set i 1} {$i < 6} {incr i} {
			radiobutton .serial.ports.commButton$i	\
				-text "COM$i"	\
				-variable serialPort	\
				-value "com$i"	
			grid .serial.ports.commButton$i -row 0 -column $i
		}
		for {set i 6} {$i < 10} {incr i} {
			radiobutton .serial.ports.commButton$i	\
				-text "COM$i"	\
				-variable serialPort	\
				-value "com$i"	
			grid .serial.ports.commButton$i -row 1 -column [expr $i-5]
		}
		
				
		#Create a radio button which allows the user to manually enter
		#a COM port.
		radiobutton .serial.ports.otherButton	\
			-text "Other:"	\
			-variable serialPort	\
			-value other
		grid .serial.ports.otherButton -row 2 -column 1
		entry .serial.ports.otherEntry	\
			-textvariable otherPort	\
			-width 6
		grid .serial.ports.otherEntry -row 2 -column 2
		
		#Create a button for the Autodetect button
		button .serial.ports.autodetect	\
			-text "Autodetect"	\
			-command {
				set autoPort [autodetect]
				.serial.ports.otherButton select
				.serial.ports.otherEntry insert 0 $autoPort
				set serialPort $autoPort
				}	\
			-width 12
		grid .serial.ports.autodetect -row 3 -column 0 -columnspan $i
	} elseif {$osType == "unix"} {
		
		#Create a list of valid port names
		for {set i 0} {$i < 4} {incr i} {
			lappend validPorts "/dev/ttyUSB$i"
		}
		
		#Create buttons for these ports
		for {set i 0} {$i < 4} {incr i} {
			radiobutton .serial.ports.ttyButton$i	\
				-text "/dev/ttyUSB$i"	\
				-variable serialPort	\
				-value "/dev/ttyUSB$i"	
			grid .serial.ports.ttyButton$i -row [expr $i] -column 0
		}
		#Radio button and entry widgets so that the user
		#can manually enter the name of a serial port
		radiobutton .serial.ports.otherButton	\
			-text "Other:"	\
			-variable serialPort	\
			-value other
		grid .serial.ports.otherButton -row [expr $i+1] -column 0
		entry .serial.ports.otherEntry	\
			-textvariable otherPort
		grid .serial.ports.otherEntry -row [expr $i+2] -column 0
		
	} elseif {$osType == "Darwin"} {
		set darwinPort [glob /dev/cu.*]
		for {set i 0} {$i < [llength $darwinPort]} {incr i} {
			radiobutton .serial.ports.cuButton$i	\
				-text [lindex $darwinPort $i]	\
				-variable serialPort	\
				-value [lindex $darwinPort $i]
			grid .serial.ports.cuButton$i -row $i -column 0
		}
		radiobutton .serial.ports.otherButton	\
			-text "Other:"	\
			-variable serialPort	\
			-value other
		incr i
		grid .serial.ports.otherButton -row $i -column 0
		entry .serial.ports.otherEntry	\
			-textvariable otherPort
		incr i
		grid .serial.ports.otherEntry -row $i -column 0
	
	}
	
	#Create a button to return to the default menu
	button .serial.back	\
		-text "Save and Exit"	\
		-width 15	\
		-command {
			set fileId [open scopeport.cfg w]
			puts $fileId $serialPort
			close $fileId
			destroy .serial
			openSerialPort
		}
	
	grid .serial.serialTitle -row 0 -column 0
	grid .serial.ports -row 1 -column 0
	grid .serial.back -row 2 -column 0
	
}